@@ -1,3 +1,4 @@
+from itertools import zip_longest
 import os
 import warnings
 
@@ -513,6 +514,16 @@ def test_reduction_names():
     assert x.mean().name.startswith("mean")
 
 
+def test_general_reduction_names():
+    dtype = int
+    a = da.reduction(
+        da.ones(10, dtype, chunks=2), np.sum, np.sum, dtype=dtype, name="foo"
+    )
+    names, tokens = list(zip_longest(*[key[0].rsplit("-", 1) for key in a.dask]))
+    assert set(names) == {"ones", "foo", "foo-partial", "foo-aggregate"}
+    assert all(tokens)
+
+
 @pytest.mark.filterwarnings("ignore:`argmax` is not implemented by dask")
 @pytest.mark.parametrize("func", [np.sum, np.argmax])
 def test_array_reduction_out(func):
@@ -678,3 +689,10 @@ def test_median(axis, keepdims, func):
         getattr(da, func)(d, axis=axis, keepdims=keepdims),
         getattr(np, func)(x, axis=axis, keepdims=keepdims),
     )
+
+
+@pytest.mark.parametrize("method", ["sum", "mean", "prod"])
+def test_object_reduction(method):
+    arr = da.ones(1).astype(object)
+    result = getattr(arr, method)().compute()
+    assert result == 1
