@@ -1,20 +1,19 @@
 import itertools
 import warnings
-from collections.abc import Mapping
 
 import numpy as np
 
 import tlz as toolz
 
-from .core import reverse_dict
-from .delayed import to_task_dask
-from .highlevelgraph import HighLevelGraph
+from .core import reverse_dict, flatten, keys_in_tasks, find_all_possible_keys
+from .delayed import unpack_collections
+from .highlevelgraph import BasicLayer, HighLevelGraph, Layer
 from .optimization import SubgraphCallable, fuse
 from .utils import ensure_dict, homogeneous_deepmap, apply
 
 
 def subs(task, substitution):
-    """ Create a new task with the values substituted
+    """Create a new task with the values substituted
 
     This is like dask.core.subs, but takes a dict of many substitutions to
     perform simultaneously.  It is not as concerned with micro performance.
@@ -52,7 +51,7 @@ def blockwise(
     dependencies=(),
     **kwargs
 ):
-    """ Create a Blockwise symbolic mutable mapping
+    """Create a Blockwise symbolic mutable mapping
 
     This is like the ``make_blockwise_graph`` function, but rather than construct a dict, it
     returns a symbolic Blockwise object.
@@ -132,8 +131,8 @@ def blockwise(
     return subgraph
 
 
-class Blockwise(Mapping):
-    """ Tensor Operation
+class Blockwise(Layer):
+    """Tensor Operation
 
     This is a lazily constructed mapping for tensor operation graphs.
     This defines a dictionary using an operation and an indexing pattern.
@@ -206,21 +205,30 @@ class Blockwise(Mapping):
     @property
     def _dict(self):
         if hasattr(self, "_cached_dict"):
-            return self._cached_dict
+            return self._cached_dict["dsk"]
         else:
             keys = tuple(map(blockwise_token, range(len(self.indices))))
             dsk, _ = fuse(self.dsk, [self.output])
             func = SubgraphCallable(dsk, self.output, keys)
-            self._cached_dict = make_blockwise_graph(
+
+            key_deps = {}
+            non_blockwise_keys = set()
+            dsk = make_blockwise_graph(
                 func,
                 self.output,
                 self.output_indices,
                 *list(toolz.concat(self.indices)),
                 new_axes=self.new_axes,
                 numblocks=self.numblocks,
-                concatenate=self.concatenate
+                concatenate=self.concatenate,
+                key_deps=key_deps,
+                non_blockwise_keys=non_blockwise_keys,
             )
-        return self._cached_dict
+            self._cached_dict = {
+                "dsk": dsk,
+                "basic_layer": BasicLayer(dsk, key_deps, non_blockwise_keys),
+            }
+        return self._cached_dict["dsk"]
 
     def __getitem__(self, key):
         return self._dict[key]
@@ -243,9 +251,17 @@ class Blockwise(Mapping):
 
         return out_d
 
+    def get_dependencies(self, all_hlg_keys):
+        _ = self._dict  # trigger materialization
+        return self._cached_dict["basic_layer"].get_dependencies(all_hlg_keys)
+
+    def cull(self, keys, all_hlg_keys):
+        _ = self._dict  # trigger materialization
+        return self._cached_dict["basic_layer"].cull(keys, all_hlg_keys)
+
 
 def make_blockwise_graph(func, output, out_indices, *arrind_pairs, **kwargs):
-    """ Tensor operation
+    """Tensor operation
 
     Applies a function, ``func``, across blocks from many different input
     collections.  We arrange the pattern with which those blocks interact with
@@ -353,6 +369,8 @@ def make_blockwise_graph(func, output, out_indices, *arrind_pairs, **kwargs):
     numblocks = kwargs.pop("numblocks")
     concatenate = kwargs.pop("concatenate", None)
     new_axes = kwargs.pop("new_axes", {})
+    key_deps = kwargs.pop("key_deps", None)
+    non_blockwise_keys = kwargs.pop("non_blockwise_keys", None)
     argpairs = list(toolz.partition(2, arrind_pairs))
 
     if concatenate is True:
@@ -417,41 +435,56 @@ def make_blockwise_graph(func, output, out_indices, *arrind_pairs, **kwargs):
         else:
             coord_maps.append(None)
             concat_axes.append(None)
+
     # Unpack delayed objects in kwargs
     dsk2 = {}
     if kwargs:
-        task, dsk2 = to_task_dask(kwargs)
+        task, dsk2 = unpack_collections(kwargs)
         if dsk2:
             kwargs2 = task
         else:
             kwargs2 = kwargs
+        if non_blockwise_keys is not None:
+            non_blockwise_keys |= find_all_possible_keys([kwargs2])
+
+    # Find all non-blockwise keys in the input arguments
+    if non_blockwise_keys is not None:
+        for arg, ind in argpairs:
+            if ind is None:
+                non_blockwise_keys |= find_all_possible_keys([arg])
 
     dsk = {}
     # Create argument lists
     for out_coords in itertools.product(*[range(dims[i]) for i in out_indices]):
+        deps = set()
         coords = out_coords + dummies
         args = []
-        for cmap, axes, arg_ind in zip(coord_maps, concat_axes, argpairs):
-            arg, ind = arg_ind
+        for cmap, axes, (arg, ind) in zip(coord_maps, concat_axes, argpairs):
             if ind is None:
                 args.append(arg)
             else:
                 arg_coords = tuple(coords[c] for c in cmap)
                 if axes:
                     tups = lol_product((arg,), arg_coords)
+                    deps.update(flatten(tups))
 
                     if concatenate:
                         tups = (concatenate, tups, axes)
                 else:
                     tups = (arg,) + arg_coords
+                    deps.add(tups)
                 args.append(tups)
+        out_key = (output,) + out_coords
+
         if kwargs:
             val = (apply, func, args, kwargs2)
         else:
             args.insert(0, func)
             val = tuple(args)
-        dsk[(output,) + out_coords] = val
+        dsk[out_key] = val
 
+        if key_deps is not None:
+            key_deps[out_key] = deps
     if dsk2:
         dsk.update(ensure_dict(dsk2))
 
@@ -459,7 +492,7 @@ def make_blockwise_graph(func, output, out_indices, *arrind_pairs, **kwargs):
 
 
 def lol_product(head, values):
-    """ List of list of tuple keys, similar to `itertools.product`.
+    """List of list of tuple keys, similar to `itertools.product`.
 
     Parameters
     ----------
@@ -487,7 +520,7 @@ def lol_product(head, values):
 
 
 def lol_tuples(head, ind, values, dummies):
-    """ List of list of tuple keys
+    """List of list of tuple keys
 
     Parameters
     ----------
@@ -526,7 +559,7 @@ def lol_tuples(head, ind, values, dummies):
 
 
 def optimize_blockwise(graph, keys=()):
-    """ High level optimization of stacked Blockwise layers
+    """High level optimization of stacked Blockwise layers
 
     For operations that have multiple Blockwise operations one after the other, like
     ``x.T + 123`` we can fuse these into a single Blockwise operation.  This happens
@@ -636,7 +669,14 @@ def _optimize_blockwise(full_graph, keys=()):
             # Merge these Blockwise layers into one
             new_layer = rewrite_blockwise([layers[l] for l in blockwise_layers])
             out[layer] = new_layer
-            dependencies[layer] = {k for k, v in new_layer.indices if v is not None}
+
+            new_deps = set()
+            for k, v in new_layer.indices:
+                if v is None:
+                    new_deps |= keys_in_tasks(full_graph.dependencies, [k])
+                else:
+                    new_deps.add(k)
+            dependencies[layer] = new_deps
         else:
             out[layer] = layers[layer]
             dependencies[layer] = full_graph.dependencies.get(layer, set())
@@ -646,7 +686,7 @@ def _optimize_blockwise(full_graph, keys=()):
 
 
 def rewrite_blockwise(inputs):
-    """ Rewrite a stack of Blockwise expressions into a single blockwise expression
+    """Rewrite a stack of Blockwise expressions into a single blockwise expression
 
     Given a set of Blockwise layers, combine them into a single layer.  The provided
     layers are expected to fit well together.  That job is handled by
@@ -806,7 +846,7 @@ def zero_broadcast_dimensions(lol, nblocks):
 
 
 def broadcast_dimensions(argpairs, numblocks, sentinels=(1, (1,)), consolidate=None):
-    """ Find block dimensions from arguments
+    """Find block dimensions from arguments
 
     Parameters
     ----------
