@@ -659,7 +659,7 @@ def _build_agg_args(spec):
     Parameters
     ----------
     spec: a list of (result-column, aggregation-function, input-column) triples.
-        To work with all arugment forms understood by pandas use
+        To work with all argument forms understood by pandas use
         ``_normalize_spec`` to normalize the argment before passing it on to
         ``_build_agg_args``.
 
@@ -668,7 +668,7 @@ def _build_agg_args(spec):
     chunk_funcs: a list of (intermediate-column, function, keyword) triples
         that are applied on grouped chunks of the initial dataframe.
 
-    agg_funcs: a list of (intermediate-column, functions, keword) triples that
+    agg_funcs: a list of (intermediate-column, functions, keyword) triples that
         are applied on the grouped concatination of the preprocessed chunks.
 
     finalizers: a list of (result-column, function, keyword) triples that are
@@ -737,6 +737,9 @@ def _build_agg_args_single(result_column, func, input_column):
     elif func == "mean":
         return _build_agg_args_mean(result_column, func, input_column)
 
+    elif func == "list":
+        return _build_agg_args_list(result_column, func, input_column)
+
     elif isinstance(func, Aggregation):
         return _build_agg_args_custom(result_column, func, input_column)
 
@@ -820,6 +823,33 @@ def _build_agg_args_mean(result_column, func, input_column):
     )
 
 
+def _build_agg_args_list(result_column, func, input_column):
+    intermediate = _make_agg_id("list", input_column)
+
+    return dict(
+        chunk_funcs=[
+            (
+                intermediate,
+                _apply_func_to_column,
+                dict(column=input_column, func=lambda s: s.apply(list)),
+            )
+        ],
+        aggregate_funcs=[
+            (
+                intermediate,
+                _apply_func_to_column,
+                dict(
+                    column=intermediate,
+                    func=lambda s0: s0.apply(
+                        lambda chunks: list(it.chain.from_iterable(chunks))
+                    ),
+                ),
+            )
+        ],
+        finalizer=(result_column, itemgetter(intermediate), dict()),
+    )
+
+
 def _build_agg_args_custom(result_column, func, input_column):
     col = _make_agg_id(funcname(func), input_column)
 
