@@ -167,7 +167,7 @@ def describe_numeric_aggregate(stats, name=None, is_timedelta_col=False):
 
     part1 = typ([count, mean, std, min], index=["count", "mean", "std", "min"])
 
-    q.index = ["{0:g}%".format(l * 100) for l in q.index.tolist()]
+    q.index = ["{0:g}%".format(l * 100) for l in tolist(q.index)]
     if is_series_like(q) and typ != type(q):
         q = q.to_frame()
     part3 = typ([max], index=["max"])
@@ -231,7 +231,7 @@ def describe_nonnumeric_aggregate(stats, name):
 
 
 def _cum_aggregate_apply(aggregate, x, y):
-    """ Apply aggregation function within a cumulative aggregation
+    """Apply aggregation function within a cumulative aggregation
 
     Parameters
     ----------
@@ -540,6 +540,19 @@ def concat_pandas(
     return out
 
 
+tolist_dispatch = Dispatch("tolist")
+
+
+def tolist(obj):
+    func = tolist_dispatch.dispatch(type(obj))
+    return func(obj)
+
+
+@tolist_dispatch.register((pd.Series, pd.Index, pd.Categorical))
+def tolist_pandas(obj):
+    return obj.tolist()
+
+
 # cuDF may try to import old dispatch functions
 hash_df = hash_object_dispatch
 group_split = group_split_dispatch
