@@ -10,26 +10,32 @@ from ..blockwise import optimize_blockwise, fuse_roots, Blockwise
 
 
 def optimize(dsk, keys, **kwargs):
+    if not isinstance(keys, (list, set)):
+        keys = [keys]
+    keys = list(core.flatten(keys))
 
-    if isinstance(dsk, HighLevelGraph):
-        # Think about an API for this.
-        flat_keys = list(core.flatten(keys))
-        dsk = optimize_read_parquet_getitem(dsk, keys=flat_keys)
-        dsk = optimize_blockwise(dsk, keys=flat_keys)
-        dsk = fuse_roots(dsk, keys=flat_keys)
+    if not isinstance(dsk, HighLevelGraph):
+        dsk = HighLevelGraph.from_collections(id(dsk), dsk, dependencies=())
 
+    dsk = optimize_read_parquet_getitem(dsk, keys=keys)
+    dsk = optimize_blockwise(dsk, keys=keys)
+    dsk = fuse_roots(dsk, keys=keys)
+    dsk = dsk.cull(set(keys))
+
+    if not config.get("optimization.fuse.active"):
+        return dsk
+
+    dependencies = dsk.get_dependencies()
     dsk = ensure_dict(dsk)
 
-    if isinstance(keys, list):
-        dsk, dependencies = cull(dsk, list(core.flatten(keys)))
-    else:
-        dsk, dependencies = cull(dsk, [keys])
-
-    dsk, dependencies = fuse(
+    fuse_subgraphs = config.get("optimization.fuse.subgraphs")
+    if fuse_subgraphs is None:
+        fuse_subgraphs = True
+    dsk, _ = fuse(
         dsk,
         keys,
         dependencies=dependencies,
-        fuse_subgraphs=config.get("fuse_subgraphs", True),
+        fuse_subgraphs=fuse_subgraphs,
     )
     dsk, _ = cull(dsk, keys)
     return dsk
